@@ -286,12 +286,15 @@ class AuthTokenResource(_AuthFederationWebSSOBase):
 
         token_id = flask.request.headers.get(
             authorization.SUBJECT_TOKEN_HEADER)
+        access_rules_support = flask.request.headers.get(
+            authorization.ACCESS_RULES_HEADER)
         allow_expired = strutils.bool_from_string(
             flask.request.args.get('allow_expired'))
         window_secs = CONF.token.allow_expired_window if allow_expired else 0
         include_catalog = 'nocatalog' not in flask.request.args
         token = PROVIDERS.token_provider_api.validate_token(
-            token_id, window_seconds=window_secs)
+            token_id, window_seconds=window_secs,
+            access_rules_support=access_rules_support)
         token_resp = render_token.render_token_response_from_model(
             token, include_catalog=include_catalog)
         resp_body = jsonutils.dumps(token_resp)
@@ -334,11 +337,20 @@ class AuthTokenResource(_AuthFederationWebSSOBase):
 class AuthFederationWebSSOResource(_AuthFederationWebSSOBase):
     @classmethod
     def _perform_auth(cls, protocol_id):
-        try:
-            remote_id_name = federation_utils.get_remote_id_parameter(
-                protocol_id)
-            remote_id = flask.request.environ[remote_id_name]
-        except KeyError:
+        idps = PROVIDERS.federation_api.list_idps()
+        remote_id = None
+        for idp in idps:
+            try:
+                remote_id_name = federation_utils.get_remote_id_parameter(
+                    idp, protocol_id)
+            except exception.FederatedProtocolNotFound:
+                # no protocol for this IdP, so this can't be the IdP we're
+                # looking for
+                continue
+            remote_id = flask.request.environ.get(remote_id_name)
+            if remote_id:
+                break
+        if not remote_id:
             msg = 'Missing entity ID from environment'
             tr_msg = _('Missing entity ID from environment')
             LOG.error(msg)
