@@ -11,10 +11,16 @@
 # under the License.
 
 from oslo_config import cfg
+from oslo_log import versionutils
 
 from keystone.conf import utils
 
 
+_DEPRECATED_MSG = utils.fmt("""
+This option has been superseded by ephemeral users existing in the domain
+of their identity provider.
+""")
+
 driver = cfg.StrOpt(
     'driver',
     default='sql',
@@ -35,15 +41,20 @@ assertions. Matched variables are passed into the federated mapping engine.
 remote_id_attribute = cfg.StrOpt(
     'remote_id_attribute',
     help=utils.fmt("""
-Value to be used to obtain the entity ID of the Identity Provider from the
-environment. For `mod_shib`, this would be `Shib-Identity-Provider`. For
-`mod_auth_openidc`, this could be `HTTP_OIDC_ISS`. For `mod_auth_mellon`, this
-could be `MELLON_IDP`.
+Default value for all protocols to be used to obtain the entity ID of the
+Identity Provider from the environment. For `mod_shib`, this would be
+`Shib-Identity-Provider`. For `mod_auth_openidc`, this could be
+`HTTP_OIDC_ISS`. For `mod_auth_mellon`, this could be `MELLON_IDP`. This can be
+overridden on a per-protocol basis by providing a `remote_id_attribute` to the
+federation protocol using the API.
 """))
 
 federated_domain_name = cfg.StrOpt(
     'federated_domain_name',
     default='Federated',
+    deprecated_for_removal=True,
+    deprecated_reason=_DEPRECATED_MSG,
+    deprecated_since=versionutils.deprecated.TRAIN,
     help=utils.fmt("""
 An arbitrary domain name that is reserved to allow federated ephemeral users to
 have a domain concept. Note that an admin will not be able to create a domain
