@@ -15,12 +15,14 @@
 import ast
 import re
 
+import flask
 import jsonschema
 from oslo_config import cfg
 from oslo_log import log
 from oslo_utils import timeutils
 import six
 
+from keystone.common import provider_api
 import keystone.conf
 from keystone import exception
 from keystone.i18n import _
@@ -28,6 +30,7 @@ from keystone.i18n import _
 
 CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
+PROVIDERS = provider_api.ProviderAPIs
 
 
 class UserType(object):
@@ -240,6 +243,10 @@ class DirectMaps(object):
     def __init__(self):
         self._matches = []
 
+    def __str__(self):
+        """return the direct map array as a string."""
+        return '%s' % self._matches
+
     def add(self, values):
         """Add a matched value to the list of matches.
 
@@ -276,23 +283,28 @@ def validate_expiration(token):
         raise exception.Unauthorized(_('Federation token is expired'))
 
 
-def get_remote_id_parameter(protocol):
+def get_remote_id_parameter(idp, protocol):
     # NOTE(marco-fargetta): Since we support any protocol ID, we attempt to
-    # retrieve the remote_id_attribute of the protocol ID. If it's not
-    # registered in the config, then register the option and try again.
-    # This allows the user to register protocols other than oidc and saml2.
-    remote_id_parameter = None
-    try:
-        remote_id_parameter = CONF[protocol]['remote_id_attribute']
-    except AttributeError:
-        # TODO(dolph): Move configuration registration to keystone.conf
-        CONF.register_opt(cfg.StrOpt('remote_id_attribute'),
-                          group=protocol)
+    # retrieve the remote_id_attribute of the protocol ID. It will look up first
+    # if the remote_id_attribute exists.
+    protocol_ref = PROVIDERS.federation_api.get_protocol(idp['id'], protocol)
+    remote_id_parameter = protocol_ref.get('remote_id_attribute')
+    if remote_id_parameter:
+        return remote_id_parameter
+    else:
+        # If it's not registered in the config, then register the option and try again.
+        # This allows the user to register protocols other than oidc and saml2.
         try:
             remote_id_parameter = CONF[protocol]['remote_id_attribute']
-        except AttributeError:  # nosec
-            # No remote ID attr, will be logged and use the default instead.
-            pass
+        except AttributeError:
+            # TODO(dolph): Move configuration registration to keystone.conf
+            CONF.register_opt(cfg.StrOpt('remote_id_attribute'),
+                              group=protocol)
+            try:
+                remote_id_parameter = CONF[protocol]['remote_id_attribute']
+            except AttributeError:  # nosec
+                # No remote ID attr, will be logged and use the default instead.
+                pass
     if not remote_id_parameter:
         LOG.debug('Cannot find "remote_id_attribute" in configuration '
                   'group %s. Trying default location in '
@@ -304,7 +316,7 @@ def get_remote_id_parameter(protocol):
 
 def validate_idp(idp, protocol, assertion):
     """The IdP providing the assertion should be registered for the mapping."""
-    remote_id_parameter = get_remote_id_parameter(protocol)
+    remote_id_parameter = get_remote_id_parameter(idp, protocol)
     if not remote_id_parameter or not idp['remote_ids']:
         LOG.debug('Impossible to identify the IdP %s ', idp['id'])
         # If nothing is defined, the administrator may want to
@@ -355,7 +367,8 @@ def transform_to_group_ids(group_names, mapping_id,
     """Transform groups identified by name/domain to their ids.
 
     Function accepts list of groups identified by a name and domain giving
-    a list of group ids in return.
+    a list of group ids in return. A message is logged if the group doesn't
+    exist in the backend.
 
     Example of group_names parameter::
 
@@ -386,9 +399,6 @@ def transform_to_group_ids(group_names, mapping_id,
 
     :returns: generator object with group ids
 
-    :raises keystone.exception.MappedGroupNotFound: in case asked group doesn't
-        exist in the backend.
-
     """
     def resolve_domain(domain):
         """Return domain id.
@@ -416,10 +426,10 @@ def transform_to_group_ids(group_names, mapping_id,
                       group['name'])
 
 
-def get_assertion_params_from_env(request):
-    LOG.debug('Environment variables: %s', request.environ)
+def get_assertion_params_from_env():
+    LOG.debug('Environment variables: %s', flask.request.environ)
     prefix = CONF.federation.assertion_prefix
-    for k, v in list(request.environ.items()):
+    for k, v in list(flask.request.environ.items()):
         if not k.startswith(prefix):
             continue
         # These bytes may be decodable as ISO-8859-1 according to Section
@@ -592,12 +602,7 @@ class RuleProcessor(object):
                 raise exception.ValidationError(msg)
 
             if user_type is None:
-                user_type = user['type'] = UserType.EPHEMERAL
-
-            if user_type == UserType.EPHEMERAL:
-                user['domain'] = {
-                    'id': CONF.federation.federated_domain_name
-                }
+                user['type'] = UserType.EPHEMERAL
 
         # initialize the group_ids as a set to eliminate duplicates
         user = {}
@@ -609,11 +614,14 @@ class RuleProcessor(object):
         # if mapping yield no valid identity values, we should bail right away
         # instead of continuing on with a normalized bogus user
         if not identity_values:
-            msg = _("Could not map any federated user properties to identity "
-                    "values. Check debug logs or the mapping used for "
-                    "additional details.")
+            msg = ("Could not map any federated user properties to identity "
+                   "values. Check debug logs or the mapping used for "
+                   "additional details.")
+            tr_msg = _("Could not map any federated user properties to "
+                       "identity values. Check debug logs or the mapping "
+                       "used for additional details.")
             LOG.warning(msg)
-            raise exception.ValidationError(msg)
+            raise exception.ValidationError(tr_msg)
 
         for identity_value in identity_values:
             if 'user' in identity_value:
@@ -686,7 +694,7 @@ class RuleProcessor(object):
 
         Example direct_maps::
 
-            ['Bob', 'Thompson', 'bob@example.com']
+            [['Bob'], ['Thompson'], ['bob@example.com']]
 
         :returns: new local mapping reference with replaced values.
 
@@ -864,14 +872,18 @@ class RuleProcessor(object):
 def assert_enabled_identity_provider(federation_api, idp_id):
     identity_provider = federation_api.get_idp(idp_id)
     if identity_provider.get('enabled') is not True:
-        msg = _('Identity Provider %(idp)s is disabled') % {'idp': idp_id}
+        msg = 'Identity Provider %(idp)s is disabled' % {
+            'idp': idp_id}
+        tr_msg = _('Identity Provider %(idp)s is disabled') % {
+            'idp': idp_id}
         LOG.debug(msg)
-        raise exception.Forbidden(msg)
+        raise exception.Forbidden(tr_msg)
 
 
 def assert_enabled_service_provider_object(service_provider):
     if service_provider.get('enabled') is not True:
         sp_id = service_provider['id']
-        msg = _('Service Provider %(sp)s is disabled') % {'sp': sp_id}
+        msg = 'Service Provider %(sp)s is disabled' % {'sp': sp_id}
+        tr_msg = _('Service Provider %(sp)s is disabled') % {'sp': sp_id}
         LOG.debug(msg)
-        raise exception.Forbidden(msg)
+        raise exception.Forbidden(tr_msg)
