@@ -26,6 +26,7 @@ import oslo_config.fixture
 from oslo_db.sqlalchemy import migration
 from oslo_log import log
 from oslo_serialization import jsonutils
+from oslo_upgradecheck import upgradecheck
 from six.moves import configparser
 from six.moves import http_client
 from six.moves import range
@@ -41,6 +42,7 @@ from keystone.cmd.doctor import ldap
 from keystone.cmd.doctor import security_compliance
 from keystone.cmd.doctor import tokens
 from keystone.cmd.doctor import tokens_fernet
+from keystone.cmd import status
 from keystone.common import provider_api
 from keystone.common.sql import upgrades
 import keystone.conf
@@ -51,6 +53,7 @@ from keystone.tests import unit
 from keystone.tests.unit import default_fixtures
 from keystone.tests.unit.ksfixtures import database
 from keystone.tests.unit.ksfixtures import ldapdb
+from keystone.tests.unit.ksfixtures import policy
 from keystone.tests.unit.ksfixtures import temporaryfile
 from keystone.tests.unit import mapping_fixtures
 
@@ -216,6 +219,12 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
             c.get('/v3/auth/tokens',
                   headers={'X-Auth-Token': r.headers['X-Subject-Token'],
                            'X-Subject-Token': token})
+        admin_role = PROVIDERS.role_api.get_role(self.bootstrap.role_id)
+        reader_role = PROVIDERS.role_api.get_role(self.bootstrap.reader_role_id)
+        member_role = PROVIDERS.role_api.get_role(self.bootstrap.member_role_id)
+        self.assertEqual(admin_role['options'], {})
+        self.assertEqual(member_role['options'], {})
+        self.assertEqual(reader_role['options'], {})
 
     def test_bootstrap_is_not_idempotent_when_password_does_change(self):
         # NOTE(lbragstad): Ensure bootstrap isn't idempotent when run with
@@ -289,6 +298,19 @@ class CliBootStrapTestCase(unit.SQLDriverOverrides, unit.TestCase):
                 user_id,
                 self.bootstrap.password)
 
+    def test_bootstrap_with_immutable_roles(self):
+        CONF(args=['bootstrap',
+                   '--bootstrap-password', uuid.uuid4().hex,
+                   '--immutable-roles'],
+             project='keystone')
+        self._do_test_bootstrap(self.bootstrap)
+        admin_role = PROVIDERS.role_api.get_role(self.bootstrap.role_id)
+        reader_role = PROVIDERS.role_api.get_role(self.bootstrap.reader_role_id)
+        member_role = PROVIDERS.role_api.get_role(self.bootstrap.member_role_id)
+        self.assertTrue(admin_role['options']['immutable'])
+        self.assertTrue(member_role['options']['immutable'])
+        self.assertTrue(reader_role['options']['immutable'])
+
     def test_bootstrap_with_ambiguous_role_names(self):
         # bootstrap system to create the default admin role
         self._do_test_bootstrap(self.bootstrap)
@@ -428,6 +450,21 @@ class CliBootStrapTestCaseWithEnvironment(CliBootStrapTestCase):
 
         self._do_test_bootstrap(self.bootstrap)
 
+    def test_endpoints_created_with_new_endpoints(self):
+        service = unit.new_service_ref(name=self.service_name, type='identity')
+        PROVIDERS.catalog_api.create_service(service['id'], service)
+        region = unit.new_region_ref(id=self.region_id)
+        PROVIDERS.catalog_api.create_region(region)
+        endpoint = unit.new_endpoint_ref(interface='public',
+                                         service_id=service['id'],
+                                         url=uuid.uuid4().hex,
+                                         region_id=self.region_id)
+        PROVIDERS.catalog_api.create_endpoint(endpoint['id'], endpoint)
+
+        self._do_test_bootstrap(self.bootstrap)
+        updated_endpoint = PROVIDERS.catalog_api.get_endpoint(endpoint['id'])
+        self.assertEqual(updated_endpoint['url'], self.bootstrap.public_url)
+
 
 class CliDomainConfigAllTestCase(unit.SQLDriverOverrides, unit.TestCase):
 
@@ -1822,9 +1859,6 @@ class TestMappingEngineTester(unit.BaseTestCase):
                 "group_names": [],
                 "user": {
                     "type": "ephemeral",
-                    "domain": {
-                        "id": "Federated"
-                    },
                     "name": "me"
                 },
                 "projects": [],
@@ -1855,3 +1889,95 @@ class TestMappingEngineTester(unit.BaseTestCase):
         mapping_engine = cli.MappingEngineTester()
         self.assertRaises(exception.ValidationError,
                           mapping_engine.main)
+
+    def test_mapping_engine_tester_logs_direct_maps(self):
+        tempfilejson = self.useFixture(temporaryfile.SecureTempFile())
+        tmpfilejsonname = tempfilejson.file_name
+        updated_mapping = copy.deepcopy(mapping_fixtures.MAPPING_SMALL)
+        with open(tmpfilejsonname, 'w') as f:
+            f.write(jsonutils.dumps(updated_mapping))
+        self.command_rules = tmpfilejsonname
+        tempfile = self.useFixture(temporaryfile.SecureTempFile())
+        tmpfilename = tempfile.file_name
+        with open(tmpfilename, 'w') as f:
+            f.write("\n")
+            f.write("UserName:me\n")
+            f.write("orgPersonType:NoContractor\n")
+            f.write("LastName:Bo\n")
+            f.write("FirstName:Jill\n")
+        self.command_input = tmpfilename
+        self.command_prefix = None
+        self.command_engine_debug = True
+        self.useFixture(fixtures.MockPatchObject(
+            CONF, 'command', self.FakeConfCommand(self)))
+        mapping_engine = cli.MappingEngineTester()
+        logging = self.useFixture(fixtures.FakeLogger(level=log.DEBUG))
+        mapping_engine.main()
+        expected_msg = "direct_maps: [['me']]"
+        self.assertThat(logging.output, matchers.Contains(expected_msg))
+
+
+class CliStatusTestCase(unit.SQLDriverOverrides, unit.TestCase):
+
+    def setUp(self):
+        self.useFixture(database.Database())
+        super(CliStatusTestCase, self).setUp()
+        self.load_backends()
+        self.policy_file = self.useFixture(temporaryfile.SecureTempFile())
+        self.policy_file_name = self.policy_file.file_name
+        self.useFixture(
+            policy.Policy(
+                self.config_fixture, policy_file=self.policy_file_name
+            )
+        )
+        self.checks = status.Checks()
+
+    def test_check_safe_trust_policies(self):
+        with open(self.policy_file_name, 'w') as f:
+            overridden_policies = {
+                'identity:list_trusts': '',
+                'identity:delete_trust': '',
+                'identity:get_trust': '',
+                'identity:list_roles_for_trust': '',
+                'identity:get_role_for_trust': ''
+            }
+            f.write(jsonutils.dumps(overridden_policies))
+        result = self.checks.check_trust_policies_are_not_empty()
+        self.assertEqual(upgradecheck.Code.FAILURE, result.code)
+        with open(self.policy_file_name, 'w') as f:
+            overridden_policies = {
+                'identity:list_trusts': 'rule:admin_required',
+                'identity:delete_trust': 'rule:admin_required',
+                'identity:get_trust': 'rule:admin_required',
+                'identity:list_roles_for_trust': 'rule:admin_required',
+                'identity:get_role_for_trust': 'rule:admin_required'
+            }
+            f.write(jsonutils.dumps(overridden_policies))
+        result = self.checks.check_trust_policies_are_not_empty()
+        self.assertEqual(upgradecheck.Code.SUCCESS, result.code)
+        with open(self.policy_file_name, 'w') as f:
+            overridden_policies = {}
+            f.write(jsonutils.dumps(overridden_policies))
+        result = self.checks.check_trust_policies_are_not_empty()
+        self.assertEqual(upgradecheck.Code.SUCCESS, result.code)
+
+    def test_check_immutable_roles(self):
+        role_ref = unit.new_role_ref(name='admin')
+        PROVIDERS.role_api.create_role(role_ref['id'], role_ref)
+        result = self.checks.check_default_roles_are_immutable()
+        self.assertEqual(upgradecheck.Code.FAILURE, result.code)
+        role_ref['options'] = {'immutable': True}
+        PROVIDERS.role_api.update_role(role_ref['id'], role_ref)
+        result = self.checks.check_default_roles_are_immutable()
+        self.assertEqual(upgradecheck.Code.SUCCESS, result.code)
+        # Check domain-specific roles are not reported
+        PROVIDERS.resource_api.create_domain(
+            default_fixtures.ROOT_DOMAIN['id'],
+            default_fixtures.ROOT_DOMAIN)
+        domain_ref = unit.new_domain_ref()
+        domain = PROVIDERS.resource_api.create_domain(
+            domain_ref['id'], domain_ref)
+        role_ref = unit.new_role_ref(name='admin', domain_id=domain['id'])
+        PROVIDERS.role_api.create_role(role_ref['id'], role_ref)
+        result = self.checks.check_default_roles_are_immutable()
+        self.assertEqual(upgradecheck.Code.SUCCESS, result.code)
