@@ -654,10 +654,8 @@ class TestCredentialAppCreds(CredentialBaseTestCase):
         Call ``POST /credentials``.
         """
         # Create the app cred
-        ref = {
-            'name': uuid.uuid4().hex,
-            'roles': [{'id': self.role_id}]
-        }
+        ref = unit.new_application_credential_ref(roles=[{'id': self.role_id}])
+        del ref['id']
         r = self.post('/users/%s/application_credentials' % self.user_id,
                       body={'application_credential': ref})
         app_cred = r.result['application_credential']
